// file: life.h

#ifndef LIFE_H
#define LIFE_H


class Cell {

public:
    Cell(int r, int c);
    ~Cell();
    int row;
    int col;
    Cell * next;
};  



class Life
{

public:
    Life(int c, int r);
    ~Life();
    bool loadList(Cell * start);
    Cell * toList();
    int getAge() const { return age; }
    int getPopulation() const { return population; }
    void randomSeed();
    int getRows() const { return rows; }
    int getCols() const { return cols; }
    void updateList();
    int countNeighbours(int h, int w);
    void checkCell(Cell * c);
    void deleteList(Cell * first);
    void nextGen();
    void clear();
    void createBBC();
    void createSlim();
    void doTrail();
    void setTrailing(bool isTrailing) { trailing = isTrailing; }
    bool getTrailing() const { return trailing; }
    void hSymmetrize();
    void vSymmetrize();
    bool pdSymmetrize();
    bool mdSymmetrize();
    bool r90Transform();
    void r180Symmetrize();
    bool r270Transform();
    void stepUp(Cell * c);
    void stepUp();
    void stepDown(Cell * c);
    void stepDown();
    void stepLeft(Cell * c);
    void stepLeft();
    void stepRight(Cell * c);
    void stepRight();
    int getMinRow(Cell * first);
    int getMaxRow(Cell * first);
    int getMinCol(Cell * first);
    int getMaxCol(Cell * first);

protected:
    int rows;
    int cols;
    int current;
    int scale;
    int population;
    int age;
    bool trailing;
    bool ** cells;
    bool ** checked;
    bool ** trail;
    Cell * start, * prior;
    void (Life::*pSurface)();
    void removeBorder();
    void setCells(Cell *);
    Cell * copyCellList(Cell *);
    void torus();
    void ppp();
    void kleinBottle();
    void mirror();
    void box();
};

#endif
