// file: lifedialog.h

#ifndef LIFEDIALOG_H
#define LIFEDIALOG_H

#include <qwidget.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qpushbutton.h>
#include <qtimer.h>
#include <qslider.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qvalidator.h>
#include <qscrollview.h>
#include <qstring.h>
#include <qcombobox.h>
#include <qinputdialog.h>
#include "life.h"
#include <qprogressdialog.h>
#include <qpopupmenu.h>
#include <qaction.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qdialog.h>

class LifeBoard : public QWidget, public Life
{
Q_OBJECT

public:
    LifeBoard(QWidget * parent, int c, int r, bool zoom = false);
    ~LifeBoard();
    void setCell(int i, int j);
    void loadFromRLE(const QString & filename);
    void saveToRLE(const QString & filename);
    void loadFromASCII(const QString & filename);
    bool getSCP() const { return showCursorPosition; }
    void setSCP(bool scp) { showCursorPosition = scp; }
    void showTrail();
    void addToEditContour(int row, int col);
    void deleteEditContour();
    void deleteEditPattern(bool inside);
    void deleteEditList();
    void moveEditPatternUp();
    void moveEditPatternDown();
    void moveEditPatternLeft();
    void moveEditPatternRight();

    void createEditList();
    void addMovedCells();

public slots:
    void surfaceSlot(int index);
    void trailSlot();
    void setEdit(bool ed) { edit = ed; }
    void setMoving(bool move) { moving = move; }
 
signals:
    void boardClicked();


protected:
    virtual void paintEvent( QPaintEvent * );
    virtual void mouseMoveEvent( QMouseEvent * );
    virtual void mousePressEvent( QMouseEvent * );
    void	 mouseHandle( const QPoint &pos );
    virtual void mouseDoubleClickEvent(QMouseEvent * );    
    virtual void leaveEvent(QEvent *);
    virtual void keyPressEvent(QKeyEvent *);

private:
    bool isInsideContour(int row, int col);

    Cell * editStart;
    Cell * contourStart, * contourLast;
    bool edit;
    bool moving;
    int scale;
    int cursorRow;
    int cursorCol;
    QPixmap * offImage;
    QPainter * offPainter;
    QWidget * parent;
    bool showCursorPosition;

};

class LifeDialog;

class MyDialog : public QWidget
{
Q_OBJECT
 public:
     MyDialog(LifeDialog * ld);

 private slots:
     void deleteClicked();
     void cancelClicked();
     void closeClicked();

 private:
     LifeBoard * board;
     QButtonGroup * buttonGroup;
     QRadioButton * radioButton1, * radioButton2;
     QPushButton * deleteButton, * cancelButton, * closeButton;

};

class LifeTimer : public QTimer
{
    Q_OBJECT
public:
    LifeTimer( QWidget * parent );
    void	pause( bool );


public slots:
    void	setSpeed( int speed );

private:
    int		interval;
};



class LifeDialog : public QWidget
{
Q_OBJECT

public:
    friend class MyDialog;
    LifeDialog();
    ~LifeDialog();
    static void showMessage(const QString & msg);
//    LifeBoard	* board;


public slots:
    void runSlot();
    void startSlot();
    void stepSlot();
    void clearSlot();
    void saveBufSlot();
    void loadBufSlot();
    void deleteBuffer(Cell *);
    void prevBufSlot();
    void rowSlot();
    void colSlot();
    void resizeSlot();
    void ageSlot();
    void randomSlot();
    void zoomSlot();
    void loadFileSlot();
    void saveFileSlot();
    void bbcSlot();
    void slimSlot();
    void prevRunSlot();
    void skipSlot();
    void symSlot();
    void hSymSlot();
    void vSymSlot();
    void hvSymSlot();
    void pdSymSlot();
    void mdSymSlot();
    void r90SymSlot();
    void r180SymSlot();
    void r270SymSlot();
    void editSlot();

private:
    QScrollView * scrollView;
    MyDialog * editDialog;
    LifeBoard	* board;
    QPushButton * runButton, * stepButton, * clearButton, * saveBufferButton;
    QPushButton * loadBufferButton, * prevBufferButton, * resizeButton;
    QPushButton * zoomButton, * randomButton, * loadFileButton;
    QPushButton * saveFileButton,* trailButton, * prevRunButton;
    QPushButton * symButton, * editButton, * bbcButton, * slimButton;
    QPushButton * skipButton;
    QLabel * speedLabel, * plusLabel, * rowLabel, * colLabel, * surfaceLabel;
    QLabel * ageLabel1, * ageLabel2, * populationLabel1, * populationLabel2;
    QLineEdit * rowField, * colField;
    QSlider * slider;
    QComboBox * combo;
    QPopupMenu * symMenu;
    QAction * hSymAct, * vSymAct, * hvSymAct, * pdSymAct;
    QAction * mdSymAct, * r90SymAct, * r180SymAct, * r270SymAct;
    Cell * buffer;
    Cell * prevBuffer;
    Cell * prevRun;
    bool running;
    bool scp;
    bool zooming;
    QIntValidator * val;
    LifeTimer * timer;
    int r, c;
};

#endif
